@@ -27,10 +27,12 @@ module Net; module SSH
   # * Port => :port
   # * PreferredAuthentications => maps to the :auth_methods option
   # * ProxyCommand => maps to the :proxy option
+  # * ProxyJump => maps to the :proxy option
   # * PubKeyAuthentication => maps to the :auth_methods option
   # * RekeyLimit => :rekey_limit
   # * User => :user
   # * UserKnownHostsFile => :user_known_hosts_file
+  # * NumberOfPasswordPrompts => :number_of_password_prompts
   #
   # Note that you will never need to use this class directly--you can control
   # whether the OpenSSH configuration files are read by passing the :config
@@ -58,8 +60,8 @@ module Net; module SSH
       # given +files+ (defaulting to the list of files returned by
       # #default_files), translates the resulting hash into the options
       # recognized by Net::SSH, and returns them.
-      def for(host, files=default_files)
-        hash = translate(files.inject({}) { |settings, file|
+      def for(host, files=expandable_default_files)
+        translate(files.inject({}) { |settings, file|
           load(file, host, settings)
         })
       end
@@ -72,11 +74,11 @@ module Net; module SSH
       # options.)
       def load(path, host, settings={})
         file = File.expand_path(path)
+        base_dir = File.dirname(file)
         return settings unless File.readable?(file)
 
         globals = {}
-        matched_host = nil
-        multi_host = []
+        host_matched = false
         seen_host = false
         IO.foreach(file) do |line|
           next if line =~ /^\s*(?:#.*)?$/
@@ -107,25 +109,35 @@ module Net; module SSH
 
             # Check for negative patterns first. If the host matches, that overrules any other positive match.
             # The host substring code is used to strip out the starting "!" so the regexp will be correct.
-            negative_match = negative_hosts.select { |h| host =~ pattern2regex(h[1..-1]) }.first
+            negative_matched = negative_hosts.any? { |h| host =~ pattern2regex(h[1..-1]) }
 
-            if negative_match
-              matched_host = nil
+            if negative_matched
+              host_matched = false
             else
-              matched_host = positive_hosts.select { |h| host =~ pattern2regex(h) }.first
+              host_matched = positive_hosts.any? { |h| host =~ pattern2regex(h) }
             end
 
             seen_host = true
             settings[key] = host
           elsif !seen_host
-            if key == 'identityfile'
+            case key
+            when 'identityfile'
               (globals[key] ||= []) << value
+            when 'include'
+              included_file_paths(base_dir, value).each do |file_path|
+                globals = load(file_path, host, globals)
+              end
             else
               globals[key] = value unless settings.key?(key)
             end
-          elsif !matched_host.nil?
-            if key == 'identityfile'
+          elsif host_matched
+            case key
+            when 'identityfile'
               (settings[key] ||= []) << value
+            when 'include'
+              included_file_paths(base_dir, value).each do |file_path|
+                settings = load(file_path, host, settings)
+              end
             else
               settings[key] = value unless settings.key?(key)
             end
@@ -144,98 +156,133 @@ module Net; module SSH
       def translate(settings)
         auth_methods = default_auth_methods.clone
         (auth_methods << 'challenge-response').uniq!
-        ret = settings.inject({:auth_methods=>auth_methods}) do |hash, (key, value)|
-          case key
-          when 'bindaddress' then
-            hash[:bind_address] = value
-          when 'ciphers' then
-            hash[:encryption] = value.split(/,/)
-          when 'compression' then
-            hash[:compression] = value
-          when 'compressionlevel' then
-            hash[:compression_level] = value
-          when 'connecttimeout' then
-            hash[:timeout] = value
-          when 'forwardagent' then
-            hash[:forward_agent] = value
-          when 'identitiesonly' then
-            hash[:keys_only] = value
-          when 'globalknownhostsfile'
-            hash[:global_known_hosts_file] = value
-          when 'hostbasedauthentication' then
-            if value
-              (hash[:auth_methods] << "hostbased").uniq!
-            else
-              hash[:auth_methods].delete("hostbased")
-            end
-          when 'hostkeyalgorithms' then
-            hash[:host_key] = value.split(/,/)
-          when 'hostkeyalias' then
-            hash[:host_key_alias] = value
-          when 'hostname' then
-            hash[:host_name] = value.gsub(/%h/, settings['host'])
-          when 'identityfile' then
-            hash[:keys] = value
-          when 'macs' then
-            hash[:hmac] = value.split(/,/)
-          when 'passwordauthentication'
-            if value
-              (hash[:auth_methods] << 'password').uniq!
-            else
-              hash[:auth_methods].delete('password')
-            end
-          when 'challengeresponseauthentication'
-            if value
-              (hash[:auth_methods] << 'challenge-response').uniq!
-            else
-              hash[:auth_methods].delete('challenge-response')
-            end
-          when 'kbdinteractiveauthentication'
-            if value
-              (hash[:auth_methods] << 'keyboard-interactive').uniq!
-            else
-              hash[:auth_methods].delete('keyboard-interactive')
-            end
-          when 'port'
-            hash[:port] = value
-          when 'preferredauthentications'
-            hash[:auth_methods] = value.split(/,/) # TODO we should place to preferred_auth_methods rather than auth_methods
-          when 'proxycommand'
-            if value and !(value =~ /^none$/)
-              require 'net/ssh/proxy/command'
-              hash[:proxy] = Net::SSH::Proxy::Command.new(value)
-            end
-	        when 'pubkeyauthentication'
-            if value
-              (hash[:auth_methods] << 'publickey').uniq!
-            else
-              hash[:auth_methods].delete('publickey')
-            end
-          when 'rekeylimit'
-            hash[:rekey_limit] = interpret_size(value)
-          when 'user'
-            hash[:user] = value
-          when 'userknownhostsfile'
-            hash[:user_known_hosts_file] = value
-          when 'sendenv'
-            multi_send_env = value.to_s.split(/\s+/)
-            hash[:send_env] = multi_send_env.map { |e| Regexp.new pattern2regex(e).source, false }
-          end
+        ret = settings.inject({auth_methods: auth_methods}) do |hash, (key, value)|
+          translate_config_key(hash, key.to_sym, value, settings)
           hash
         end
         merge_challenge_response_with_keyboard_interactive(ret)
       end
 
+      # Filters default_files down to the files that are expandable.
+      def expandable_default_files
+        default_files.keep_if do |path|
+          begin
+            File.expand_path(path)
+            true
+          rescue ArgumentError
+            false
+          end
+        end
+      end
+
       private
 
+        def translate_config_key(hash, key, value, settings)
+          rename = {
+            bindaddress: :bind_address,
+            compression: :compression,
+            compressionlevel: :compression_level,
+            connecttimeout: :timeout,
+            forwardagent: :forward_agent,
+            identitiesonly: :keys_only,
+            globalknownhostsfile: :global_known_hosts_file,
+            hostkeyalias: :host_key_alias,
+            identityfile: :keys,
+            port: :port,
+            user: :user,
+            userknownhostsfile: :user_known_hosts_file
+          }
+          case key
+            when :ciphers
+              hash[:encryption] = value.split(/,/)
+            when :hostbasedauthentication
+              if value
+                (hash[:auth_methods] << "hostbased").uniq!
+              else
+                hash[:auth_methods].delete("hostbased")
+              end
+            when :hostkeyalgorithms
+              hash[:host_key] = value.split(/,/)
+            when :hostname
+              hash[:host_name] = value.gsub(/%h/, settings['host'])
+            when :macs
+              hash[:hmac] = value.split(/,/)
+            when :serveralivecountmax
+              hash[:keepalive_maxcount] = value.to_i if value
+            when :serveraliveinterval
+              if value && value.to_i > 0
+                hash[:keepalive] = true
+                hash[:keepalive_interval] = value.to_i
+              else
+                hash[:keepalive] = false
+              end
+            when :passwordauthentication
+              if value
+                (hash[:auth_methods] << 'password').uniq!
+              else
+                hash[:auth_methods].delete('password')
+              end
+            when :challengeresponseauthentication
+              if value
+                (hash[:auth_methods] << 'challenge-response').uniq!
+              else
+                hash[:auth_methods].delete('challenge-response')
+              end
+            when :kbdinteractiveauthentication
+              if value
+                (hash[:auth_methods] << 'keyboard-interactive').uniq!
+              else
+                hash[:auth_methods].delete('keyboard-interactive')
+              end
+            when :preferredauthentications
+              hash[:auth_methods] = value.split(/,/) # TODO we should place to preferred_auth_methods rather than auth_methods
+            when :proxycommand
+              if value and !(value =~ /^none$/)
+                require 'net/ssh/proxy/command'
+                hash[:proxy] = Net::SSH::Proxy::Command.new(value)
+              end
+            when :proxyjump
+              if value
+                require 'net/ssh/proxy/jump'
+                hash[:proxy] = Net::SSH::Proxy::Jump.new(value)
+              end
+            when :pubkeyauthentication
+              if value
+                (hash[:auth_methods] << 'publickey').uniq!
+              else
+                hash[:auth_methods].delete('publickey')
+              end
+            when :rekeylimit
+              hash[:rekey_limit] = interpret_size(value)
+            when :sendenv
+              multi_send_env = value.to_s.split(/\s+/)
+              hash[:send_env] = multi_send_env.map { |e| Regexp.new pattern2regex(e).source, false }
+            when :numberofpasswordprompts
+              hash[:number_of_password_prompts] = value.to_i
+            when *rename.keys
+              hash[rename[key]] = value
+          end
+        end
+
         # Converts an ssh_config pattern into a regex for matching against
         # host names.
         def pattern2regex(pattern)
-          pattern = "^" + pattern.to_s.gsub(/\./, "\\.").
-            gsub(/\?/, '.').
-            gsub(/([+\/])/, '\\\\\\0').
-            gsub(/\*/, '.*') + "$"
-          Regexp.new(pattern, true)
+          tail = pattern
+          prefix = ""
+          while !tail.empty? do
+            head,sep,tail = tail.partition(/[\*\?]/)
+            prefix = prefix + Regexp.quote(head)
+            case sep
+            when '*'
+              prefix += '.*'
+            when '?'
+              prefix += '.'
+            when ''
+            else
+              fail "Unpexpcted sep:#{sep}"
+            end
+          end
+          Regexp.new("^" + prefix + "$", true)
         end
 
         # Converts the given size into an integer number of bytes.
@@ -247,7 +294,7 @@ module Net; module SSH
           else size.to_i
           end
         end
-        
+
         def merge_challenge_response_with_keyboard_interactive(hash)
           if hash[:auth_methods].include?('challenge-response')
             hash[:auth_methods].delete('challenge-response')
@@ -255,6 +302,13 @@ module Net; module SSH
           end
           hash
         end
+
+        def included_file_paths(base_dir, config_path)
+          paths = Dir.glob(File.expand_path(config_path)).select { |f| File.file?(f) }
+          paths += Dir.glob(File.join(base_dir, config_path)).select { |f| File.file?(f) }
+          paths.uniq
+        end
+
     end
   end
 
