@@ -14,14 +14,14 @@
 #
 # http://net-ssh.lighthouseapp.com/projects/36253/tickets/7
 
-require_relative './common'
+require_relative 'common'
 require 'net/ssh/buffer'
 require 'net/ssh'
 require 'net/ssh/proxy/command'
 require 'timeout'
 require 'tempfile'
 
-class TestForward < Test::Unit::TestCase
+class ForwardTestBase < NetSSHTest
   include IntegrationTestHelpers
 
   def localhost
@@ -33,14 +33,14 @@ class TestForward < Test::Unit::TestCase
   end
 
   def ssh_start_params(options = {})
-    [localhost ,user , {:keys => @key_id_rsa}.merge(options)]
+    [localhost ,user , {keys: @key_id_rsa}.merge(options)]
   end
 
   def setup_ssh_env(&block)
     tmpdir do |dir|
       @key_id_rsa = "#{dir}/id_rsa"
       sh "rm -rf #{@key_id_rsa} #{@key_id_rsa}.pub"
-      sh "ssh-keygen -f #{@key_id_rsa} -t rsa -N ''"
+      sh "ssh-keygen -q -f #{@key_id_rsa} -t rsa -N ''"
       set_authorized_key(user,"#{@key_id_rsa}.pub")
       yield
     end
@@ -65,7 +65,9 @@ class TestForward < Test::Unit::TestCase
     end
     return server
   end
+end
 
+class TestForward < ForwardTestBase
   def start_server_closing_soon(exceptions=nil)
     server = TCPServer.open(0)
     Thread.start do
@@ -88,7 +90,6 @@ class TestForward < Test::Unit::TestCase
   def test_in_file_no_password
     setup_ssh_env do
       ret = Net::SSH.start(*ssh_start_params) do |ssh|
-      #ret = Net::SSH.start("localhost", "net_ssh_1", {keys: @key_id_rsa}) do |ssh|
         ssh.exec! 'echo "hello from:$USER"'
       end
       assert_equal "hello from:net_ssh_1\n", ret
@@ -225,45 +226,6 @@ class TestForward < Test::Unit::TestCase
     end
   end
 
-  def create_local_socket(&blk)
-    tempfile = Tempfile.new("net_ssh_forward_test")
-    path = tempfile.path
-    tempfile.delete
-    yield UNIXServer.open(path)
-    File.delete(path)
-  end if defined?(UNIXServer)
-
-  def test_forward_local_unix_socket_to_remote_port
-    setup_ssh_env do
-      session = Net::SSH.start(*ssh_start_params)
-      server_exc = Queue.new
-      server = start_server_sending_lot_of_data(server_exc)
-      remote_port = server.addr[1]
-      client_data = nil
-
-      create_local_socket do |local_socket|
-        session.forward.local(local_socket, localhost, remote_port)
-        client_done = Queue.new
-
-        Thread.start do
-          begin
-            client = UNIXSocket.new(local_socket.path)
-            client_data = client.recv(1024)
-            client.close
-            sleep(0.2)
-          ensure
-            client_done << true
-          end
-        end
-
-        session.loop(0.1) { client_done.empty? }
-      end
-
-      assert_not_nil(client_data, "client should have received data")
-      assert(client_data.match(/item\d/), 'client should have received the string item')
-    end
-  end if defined?(UNIXSocket)
-
   def test_loop_should_not_abort_when_server_side_of_forward_is_closed
     setup_ssh_env do
       session = Net::SSH.start(*ssh_start_params)
@@ -317,7 +279,7 @@ class TestForward < Test::Unit::TestCase
       client_done = Queue.new
       got_remote_port = Queue.new
       local_port = server.addr[1]
-      session.forward.remote(0, localhost, local_port) do |actual_remote_port|
+      session.forward.remote(0, localhost, local_port, localhost) do |actual_remote_port|
         got_remote_port << actual_remote_port
       end
       session.loop(0.1) { got_remote_port.empty? }
@@ -332,7 +294,7 @@ class TestForward < Test::Unit::TestCase
           client_done << $!
         end
       end
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { server_done.empty? }
         assert_equal message, server_done.pop
       end
@@ -384,10 +346,12 @@ class TestForward < Test::Unit::TestCase
           client_done << $!
         end
       end
+      server_error = nil
       Timeout.timeout(5) do
         begin
           session.loop(0.1) { true }
-        rescue EOFError, IOError
+        rescue IOError, Errno::EBADF
+          server_error = $!
           #puts "Error: #{$!} #{$!.backtrace.join("\n")}"
         end
       end
@@ -470,7 +434,7 @@ class TestForward < Test::Unit::TestCase
           client_done << $!
         end
       end
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { server_done.empty? }
         assert_equal message, server_done.pop
       end
@@ -488,7 +452,7 @@ class TestForward < Test::Unit::TestCase
       client_done = Queue.new
       got_remote_port = Queue.new
       local_port = server.addr[1]
-      session.forward.remote(0, localhost, local_port) do |actual_remote_port|
+      session.forward.remote(0, localhost, local_port, localhost) do |actual_remote_port|
         got_remote_port << actual_remote_port
       end
       session.loop(0.1) { got_remote_port.empty? }
@@ -503,7 +467,7 @@ class TestForward < Test::Unit::TestCase
           client_done << $!
         end
       end
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
         assert_equal message, client_done.pop
       end
@@ -531,7 +495,7 @@ class TestForward < Test::Unit::TestCase
           client_done << $!
         end
       end
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
         assert_equal message, client_done.pop
       end
@@ -555,7 +519,7 @@ class TestForward < Test::Unit::TestCase
     setup_ssh_env do
       session = Net::SSH.start(*ssh_start_params)
       server = start_server do |client|
-        data = client.write "hello"
+        _data = client.write "hello"
         client.close
       end
       # Forward to a non existing port
@@ -564,16 +528,16 @@ class TestForward < Test::Unit::TestCase
       # should return connection refused
       client_done = Queue.new
       _run_reading_client(client_done, local_port)
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
       end
-      assert_equal nil, client_done.pop
+      assert_nil client_done.pop
       assert client_done.empty?
       # Forward to existing port
       remote_port = server.addr[1]
       local_port = session.forward.local(0, localhost, remote_port)
       _run_reading_client(client_done, local_port)
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
       end
       assert_equal "hello", client_done.pop
@@ -592,19 +556,19 @@ class TestForward < Test::Unit::TestCase
       # should return connection refused
       client_done = Queue.new
       _run_reading_client(client_done, local_port)
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
       end
-      assert_equal nil, client_done.pop
+      assert_nil client_done.pop
       assert client_done.empty?
       # start server
       server = TCPServer.open(remote_port)
       server = start_server(server) do |client|
-        data = client.write "hello"
+        _data = client.write "hello"
         client.close
       end
       _run_reading_client(client_done, local_port)
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
       end
       assert_equal "hello", client_done.pop
@@ -616,7 +580,7 @@ class TestForward < Test::Unit::TestCase
     setup_ssh_env do
       session = Net::SSH.start(*ssh_start_params)
       server = start_server(server) do |client|
-        data = client.write "hello"
+        _data = client.write "hello"
         client.close
       end
       remote_port = server.addr[1]
@@ -624,7 +588,7 @@ class TestForward < Test::Unit::TestCase
       # run client
       client_done = Queue.new
       _run_reading_client(client_done, local_port)
-      timeout(5) do
+      Timeout.timeout(5) do
         session.loop(0.1) { client_done.empty? }
       end
       assert_equal "hello", client_done.pop
@@ -635,3 +599,49 @@ class TestForward < Test::Unit::TestCase
     end
   end
 end
+
+class TestForwardOnUnixSockets < ForwardTestBase
+  if defined?(UNIXServer) && defined?(UNIXSocket)
+    def create_local_socket(&blk)
+      tempfile = Tempfile.new("net_ssh_forward_test")
+      path = tempfile.path
+      tempfile.delete
+      yield UNIXServer.open(path)
+      File.delete(path)
+    end
+
+    def test_forward_local_unix_socket_to_remote_port
+      setup_ssh_env do
+        session = Net::SSH.start(*ssh_start_params)
+        server_exc = Queue.new
+        server = start_server_sending_lot_of_data(server_exc)
+        remote_port = server.addr[1]
+        client_data = nil
+
+        create_local_socket do |local_socket|
+          session.forward.local(local_socket, localhost, remote_port)
+          client_done = Queue.new
+
+          Thread.start do
+            begin
+              client = UNIXSocket.new(local_socket.path)
+              client_data = client.recv(1024)
+              client.close
+              sleep(0.2)
+            ensure
+              client_done << true
+            end
+          end
+
+          begin
+            session.loop(0.1) { client_done.empty? }
+          rescue Errno::EPIPE
+          end
+        end
+
+        assert_not_nil(client_data, "client should have received data")
+        assert(client_data.match(/item\d/), 'client should have received the string item')
+      end
+    end
+  end
+end
