@@ -1,9 +1,10 @@
-require 'common'
+require_relative './common'
 require 'net/ssh/config'
+require 'net/ssh'
 require 'pathname'
 require 'tempfile'
 
-class TestConfig < Test::Unit::TestCase
+class TestConfig < NetSSHTest
   def test_home_should_be_absolute_path
     assert Pathname.new(ENV['HOME']).absolute?
   end
@@ -62,7 +63,7 @@ class TestConfig < Test::Unit::TestCase
     }
     with_config_from_data data do |f|
       config = Net::SSH::Config.load(f, "test.host")
-      assert_equal nil, config['port']
+      assert_nil config['port']
       config = Net::SSH::Config.load(f, "|")
       assert_equal 1234, config['port']
     end
@@ -77,20 +78,20 @@ class TestConfig < Test::Unit::TestCase
     assert_equal %w(~/.ssh/id_dsa), config[:keys]
     assert !config.key?(:rekey_limit)
   end
-  
+
   def test_load_with_no_host
     config = Net::SSH::Config.load(config(:nohost), "test.host")
     assert_equal %w(~/.ssh/id_dsa ~/.ssh/id_rsa), config['identityfile']
     assert_equal 1985, config['port']
   end
-  
+
   def test_load_with_multiple_hosts
     config = Net::SSH::Config.load(config(:multihost), "test.host")
     assert config['compression']
     assert_equal '2G', config['rekeylimit']
     assert_equal 1980, config['port']
   end
-  
+
   def test_load_with_multiple_hosts_and_config_should_match_for_both
     aconfig = Net::SSH::Config.load(config(:multihost), "test.host")
     bconfig = Net::SSH::Config.load(config(:multihost), "other.host")
@@ -98,7 +99,7 @@ class TestConfig < Test::Unit::TestCase
     assert_equal aconfig['compression'], bconfig['compression']
     assert_equal aconfig['rekeylimit'], bconfig['rekeylimit']
   end
-  
+
   def test_load_should_parse_equal_sign_delimiters
     config = Net::SSH::Config.load(config(:eqsign), "test.test")
     assert config['compression']
@@ -189,7 +190,7 @@ class TestConfig < Test::Unit::TestCase
     net_ssh = Net::SSH::Config.translate(open_ssh)
     assert_equal %w(keyboard-interactive none password publickey), net_ssh[:auth_methods].sort
   end
-  
+
   def test_should_ddisable_keyboard_interactive_when_challeng_and_keyboardinteractive_is_off
     open_ssh = {
       'challengeresponseauthentication' => false,
@@ -209,12 +210,28 @@ class TestConfig < Test::Unit::TestCase
     config = Net::SSH::Config.for("test.host", [config(:empty), config(:auth_on), config(:auth_off)])
     assert_equal %w(hostbased keyboard-interactive none password publickey), config[:auth_methods].sort
   end
-  
+
+  def test_configuration_for_when_HOME_is_null_should_not_raise
+    with_home_env(nil) do
+      with_restored_default_files do
+        Net::SSH.configuration_for("test.host", true)
+      end
+    end
+  end
+
+  def test_config_for_when_HOME_is_null_should_not_raise
+    with_home_env(nil) do
+      with_restored_default_files do
+        Net::SSH::Config.for("test.host")
+      end
+    end
+  end
+
   def test_load_with_plus_sign_hosts
     config = Net::SSH::Config.load(config(:host_plus), "test.host")
     assert config['compression']
   end
-  
+
   def test_load_with_numeric_host
     config = Net::SSH::Config.load(config(:numeric_host), "1234")
     assert config['compression']
@@ -252,8 +269,34 @@ class TestConfig < Test::Unit::TestCase
     assert net_ssh[:proxy]
   end
 
+  def test_load_with_proxy_jump
+    config = Net::SSH::Config.load(config(:proxy_jump), "behind-proxy")
+    net_ssh = Net::SSH::Config.translate(config)
+    assert net_ssh[:proxy]
+  end
+
+  def test_load_with_include_keyword
+    config = Net::SSH::Config.load(config(:include), "xyz")
+    net_ssh = Net::SSH::Config.translate(config)
+    assert_equal 'example.com', net_ssh[:host_name]
+    assert_equal 'foo', net_ssh[:user]
+    assert_equal 2345, net_ssh[:port]
+    assert net_ssh[:keys_only]
+    assert_equal %w(~/.ssh/id.pem), net_ssh[:keys]
+  end
+
   private
 
+    def with_home_env(value,&block)
+      env_home_before = ENV['HOME']
+      begin
+        ENV['HOME'] = value
+        yield
+      ensure
+        ENV['HOME'] = env_home_before
+      end
+    end
+
     def config(name)
       "test/configs/#{name}"
     end
